// (C) 2007-2025 GoodData Corporation
import React, { useState, useRef, useEffect, useCallback } from "react";
import { injectIntl } from "react-intl";
import { StartDateInputField } from "./StartDateInputField.js";
import { EndDateInputField } from "./EndDateInputField.js";
import { DateRangeHint } from "./DateRangeHint.js";
import { DayPicker } from "./DatePicker.js";
import { DATE_INPUT_HINT_ID, TIME_INPUT_HINT_ID } from "./types.js";
const isClickOutsideOfCalendar = (event, container, inputFrom, inputTo) => {
    return (container &&
        !container.contains(event.target) &&
        inputFrom &&
        !inputFrom.contains(event.target) &&
        inputTo &&
        !inputTo.contains(event.target));
};
const setTimeForDate = (date, time) => {
    const result = new Date(date);
    result.setHours(time.getHours());
    result.setMinutes(time.getMinutes());
    return result;
};
const DateRangePickerComponent = ({ range, onRangeChange, errors, dateFormat, dayPickerProps, intl, isMobile, isTimeEnabled, weekStart = "Sunday", shouldOverlayDatePicker = false, }) => {
    const [isOpen, setIsOpen] = useState(false);
    const [inputFromValue, setInputFromValue] = useState(range.from);
    const [inputToValue, setInputToValue] = useState(range.to);
    const [selectedRange, setSelectedRange] = useState({ from: range.from, to: range.to });
    const [selectedInput, setSelectedInput] = useState(null);
    const dateRangePickerInputFrom = useRef(null);
    const dateRangePickerInputTo = useRef(null);
    const dateRangePickerContainer = useRef(null);
    // Update internal state of inputs to match the current range. This is necessary so the inputs are
    // synchronized when the range was changed from the other set of inputs, i.e., "from" calendar set
    // both "from" and "to". If both inputs were empty, one of them would remain empty even when the calendar
    // set both of them and Date filter knows about both dates from the range.
    useEffect(() => {
        setInputFromValue(range.from);
        setInputToValue(range.to);
    }, [range]);
    const handleRangeSelect = useCallback((_range, selectedDate) => {
        let calculatedFrom;
        let calculatedTo;
        if (selectedInput === "from") {
            calculatedFrom =
                inputFromValue === undefined
                    ? selectedDate
                    : setTimeForDate(selectedDate, inputFromValue);
            calculatedTo = inputToValue;
        }
        else {
            calculatedFrom = inputFromValue;
            calculatedTo =
                inputToValue === undefined ? selectedDate : setTimeForDate(selectedDate, inputToValue);
        }
        setInputFromValue(calculatedFrom);
        setInputToValue(calculatedTo);
        setSelectedRange({ from: calculatedFrom, to: calculatedTo });
        setIsOpen(false);
        onRangeChange({ from: calculatedFrom, to: calculatedTo }, { rangePosition: selectedInput });
    }, [inputFromValue, inputToValue, onRangeChange, selectedInput]);
    const handleFromChange = useCallback((date, parseError) => {
        if (date) {
            setInputFromValue(date);
        }
        setSelectedRange((prevRange) => ({ from: date, to: prevRange.to }));
        onRangeChange({ from: date, to: selectedRange.to }, { rangePosition: "from", parseError });
    }, [onRangeChange, selectedRange.to]);
    const handleToChange = useCallback((date, parseError) => {
        if (date) {
            setInputToValue(date);
        }
        setSelectedRange((prevRange) => ({ from: prevRange.from, to: date }));
        onRangeChange({ from: selectedRange.from, to: date }, { rangePosition: "to", parseError });
    }, [onRangeChange, selectedRange.from]);
    const handleFromDayClick = useCallback(() => {
        setSelectedInput("from");
        setIsOpen(true);
    }, []);
    const handleToDayClick = useCallback(() => {
        setSelectedInput("to");
        setIsOpen(true);
    }, []);
    const onKeyDown = useCallback((e) => {
        if (e.key === "Escape" || e.key === "Tab") {
            setIsOpen(false);
        }
    }, []);
    const handleClickOutside = useCallback((event) => {
        if (isClickOutsideOfCalendar(event, dateRangePickerContainer.current, dateRangePickerInputFrom.current, dateRangePickerInputTo.current)) {
            setIsOpen(false);
        }
    }, []);
    useEffect(() => {
        document.addEventListener("mousedown", handleClickOutside);
        return () => document.removeEventListener("mousedown", handleClickOutside);
    }, [handleClickOutside]);
    const onFromInputMarkedValid = useCallback((newStartDate, previousEndDate) => {
        onRangeChange({ from: newStartDate, to: previousEndDate }, { rangePosition: "from" });
    }, [onRangeChange]);
    const onToInputMarkedValid = useCallback((previousStartDate, newEndDate) => {
        onRangeChange({ from: previousStartDate, to: newEndDate }, { rangePosition: "to" });
    }, [onRangeChange]);
    const StartDateField = (React.createElement(StartDateInputField, { ref: dateRangePickerInputFrom, value: inputFromValue, onKeyDown: onKeyDown, onChange: handleFromChange, onInputMarkedValid: (date) => onFromInputMarkedValid(date, selectedRange.to), onInputClick: handleFromDayClick, errors: errors?.from, dateFormat: dateFormat, isMobile: isMobile, isTimeEnabled: isTimeEnabled, intl: intl }));
    const EndDateField = (React.createElement(EndDateInputField, { ref: dateRangePickerInputTo, value: inputToValue, onKeyDown: onKeyDown, onChange: handleToChange, onInputMarkedValid: (date) => onToInputMarkedValid(selectedRange.from, date), onInputClick: handleToDayClick, errors: errors?.to, dateFormat: dateFormat, isMobile: isMobile, isTimeEnabled: isTimeEnabled, intl: intl }));
    const isFromInputDatePickerOpen = selectedInput === "from" && isOpen;
    const isToInputDatePickerOpen = selectedInput === "to" && isOpen;
    const DatePicker = (React.createElement(DayPicker, { ref: dateRangePickerContainer, mode: selectedInput, originalDateRange: selectedRange, selectedDateRange: selectedRange, onDateRangeSelect: handleRangeSelect, alignTo: `.gd-date-range-picker-${isTimeEnabled ? selectedInput : "from"}`, calendarClassNames: `gd-date-range-picker-picker s-date-range-calendar-${selectedInput}`, dayPickerProps: dayPickerProps, weekStart: weekStart, renderAsOverlay: shouldOverlayDatePicker, intl: intl }));
    return (React.createElement(React.Fragment, null,
        isTimeEnabled ? (React.createElement("div", { className: "gd-date-range-picker datetime s-date-range-picker" },
            StartDateField,
            isFromInputDatePickerOpen ? DatePicker : null,
            EndDateField,
            isToInputDatePickerOpen ? DatePicker : null)) : (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "gd-date-range-picker gd-date-range-row s-date-range-picker" },
                StartDateField,
                EndDateField),
            isOpen ? DatePicker : null)),
        isMobile ? null : (React.createElement(DateRangeHint, { dateFormat: dateFormat, isTimeEnabled: isTimeEnabled, dateHintId: DATE_INPUT_HINT_ID, timeHintId: TIME_INPUT_HINT_ID, intl: intl }))));
};
export const DateRangePicker = injectIntl(DateRangePickerComponent);
//# sourceMappingURL=DateRangePicker.js.map